/******************************************************************************
  Copyright (c) 2006,2008,2009 Turku PET Centre

  File:        decpoint.c
  Description: Contains functions for reading real numbers from strings which
               may contain either decimal dots or commas.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  See the GNU Lesser General Public License for more details:
  http://www.gnu.org/copyleft/lesser.html

  You should have received a copy of the GNU Lesser General Public License
  along with this library/program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

  Turku PET Centre, Turku, Finland, http://www.turkupetcentre.fi

  Modification history:
  2006-06-03 Vesa Oikonen
    First created.
  2008-10-10 VO
    Added function to calculate nr of decimals, dec_nr()).
  2009-06-09 VO
    Added function atof_with_check() although this does not directly relate to
    decimal points.


******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
/*****************************************************************************/
#include "include/decpoint.h"
/*****************************************************************************/

/*****************************************************************************/
/** Checks whether argument string contains a decimal comma instead of dot.
\return Returns 1 if decimal comma is found and 0 if not found.
 */
int dec_comma_is(char *str)
{
  char *cptr=str;
  if(strchr(str, '.')!=NULL) return(0);
  while(cptr!=NULL) {if(*cptr==',') return(1); cptr++;}
  return(atof(str));
}
/*****************************************************************************/

/*****************************************************************************/
/** Replacement of atof(), which works whether string contains decimal dots
 *  or decimal commas. Possible commas are replaced by dots in the
 *  argument string.
 */
double atof_dpi(char *str)
{
  char *cptr;

  /* If string contains a dot, then use atof directly */
  if(strchr(str, '.')!=NULL) return(atof(str));
  /* Otherwise, convert all commas to dots */
  while((cptr=strchr(str, ','))!=NULL) *cptr='.';
  return(atof(str));
}
/*****************************************************************************/

/*****************************************************************************/
/** Returns the number of decimapl places in the argument string, representing
 *  a floating point value. String can contain either decimal dots or commas.
 */
int dec_nr(char *str)
{
  char *cptr;
  int n;

  /* Find the first dot or comma, but stop with E */
  cptr=str;
  while(*cptr!='.' && *cptr!=',') {
    if(*cptr==(char)0 || *cptr=='E' || *cptr=='e') return 0;
    cptr++;
  }
  if(*cptr==(char)0 || (*cptr!='.' && *cptr!=',')) return 0;
  /* Calculate the number of digits that follows */
  cptr++; n=0; while(cptr!=NULL && isdigit(*cptr)) {n++; cptr++;}
  return n;
}
/*****************************************************************************/

/*****************************************************************************/
/** Converts a string to float using strtod(), but if its return
    value is zero this function checks that argument string actually contains
    a number.
\return Returns 0 if successful, and 1 in case of an error.
*/
int atof_with_check(
  /** String which is converted to a double */
  char *double_as_string,
  /** Pointer to the double float */
  double *result_value
) {
  char* cptr;

  *result_value=atof_dpi(double_as_string);
  if(*result_value!=0.0) return(0);
  cptr=double_as_string;
  while(*cptr!=0 && (*cptr=='+' || *cptr=='-' || *cptr==' ')) cptr++;
  if(*cptr=='0') return(0); else return(1);
}
/*****************************************************************************/

/*****************************************************************************/
